@@ -1,7 +1,7 @@
 """
 PySCeS - Python Simulator for Cellular Systems (http://pysces.sourceforge.net)
 
-Copyright (C) 2004-2013 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
+Copyright (C) 2004-2022 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
 
 Brett G. Olivier (bgoli@users.sourceforge.net)
 Triple-J Group for Molecular Cell Physiology
@@ -14,8 +14,12 @@ this distribution for specifics.
 NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
 Brett G. Olivier
 """
+from __future__ import division, print_function
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
 from pysces.version import __version__
+
 __doc__ = '''
           PyscesLink
           ----------
@@ -23,13 +27,41 @@ __doc__ = '''
           Interfaces to external software and API's, has replaced the PySCeS contrib classes.
           '''
 
+try:
+    input = raw_input  # Py2 compatibility
+except NameError:
+    pass
+
 # for METATOOLlink
-import os, re, cStringIO
+import os, re, io
+
 # for SBWWebLink
-import urllib, urllib2, getpass
+try:
+    from urllib.request import (
+        ProxyHandler,
+        build_opener,
+        HTTPHandler,
+        install_opener,
+        urlopen,
+    )  # Py 3
+except ImportError:
+    from urllib2 import (
+        ProxyHandler,
+        build_opener,
+        HTTPHandler,
+        install_opener,
+        urlopen,
+    )  # Py 2
+
+try:
+    from urllib.parse import urlencode  # Py 3
+except ImportError:
+    from urllib import urlencode  # Py 2
+
 
 class SBWlink(object):
     """Generic access for local SBW services using SBWPython """
+
     sbw = None
     psbw = None
     sbwModuleProxy = None
@@ -40,6 +72,7 @@ class SBWlink(object):
         try:
             import SBW as SBW
             import SBW.psbw as psbw
+
             ##  reload(SBW)
             ##  reload(psbw)
             self.sbw = SBW
@@ -52,10 +85,10 @@ class SBWlink(object):
                     self.SBW_exposeAll(self.moduleDict[m])
                     self.modules.append(self.moduleDict[m].pythonName)
                     setattr(self, self.moduleDict[m].pythonName, self.moduleDict[m])
-            print '\nSBWlink established.'
-        except Exception, ex:
-            print ex
-            print '\nSBWlink not established.'
+            print('\nSBWlink established.')
+        except Exception as ex:
+            print(ex)
+            print('\nSBWlink not established.')
 
     def SBW_exposeAll(self, module):
         for s in module.services:
@@ -69,10 +102,10 @@ class SBWlink(object):
         for id in self.psbw.getModuleIdList():
             idlist.append(id)
             namelst.append(self.psbw.getModuleName(id))
-        for id in self.moduleDict.keys():
+        for id in list(self.moduleDict.keys()):
             if id not in idlist:
                 self.moduleDict.pop(id)
-        for name in range(len(self.modules)-1,-1,-1):
+        for name in range(len(self.modules) - 1, -1, -1):
             if self.modules[name] not in namelst:
                 delattr(self, self.modules[name])
                 self.modules.pop(name)
@@ -84,24 +117,34 @@ class SBWlink(object):
     def SBW_loadModule(self, module_name):
         ans = 'Y'
         if module_name[-3:] == 'GUI':
-            ans = raw_input('Warning! This may hang the console\n\yPress \'Y\' to continue: ')
+            ans = input(
+                'Warning! This may hang the console\n\yPress \'Y\' to continue: '
+            )
         if ans == 'Y':
             module_id = self.psbw.SBWGetModuleInstance(module_name)
             assert module_id != None, '\nUnknow module, %s' % module_name
             module = self.sbwModuleProxy.ModuleProxy(module_id)
             self.SBW_exposeAll(module)
-            if not self.moduleDict.has_key(module_id):
-                print '<PySCeS_SBW> Adding ' + module.pythonName + ' to ModuleProxy (id=' + str(module_id) + ')'
-                self.moduleDict.update({module_id : module})
+            if module_id not in self.moduleDict:
+                print(
+                    '<PySCeS_SBW> Adding '
+                    + module.pythonName
+                    + ' to ModuleProxy (id='
+                    + str(module_id)
+                    + ')'
+                )
+                self.moduleDict.update({module_id: module})
             if module.pythonName not in self.modules:
-                print '<PySCeS_SBW> Adding ' + module.pythonName + ' to SBWlink'
+                print('<PySCeS_SBW> Adding ' + module.pythonName + ' to SBWlink')
                 self.modules.append(module.pythonName)
                 setattr(self, module.pythonName, module)
         else:
-            print '\nModule %s not loaded' % module_name
+            print('\nModule %s not loaded' % module_name)
+
 
 class SBWLayoutWebLink(object):
     """Enables access to DrawNetwork and SBMLLayout web services at www.sys-bio.org"""
+
     sbwhost = '128.208.17.26'
     sbml = None
     sbmllayout = None
@@ -122,15 +165,16 @@ class SBWLayoutWebLink(object):
         port = 3128
         """
         proxy_info = {}
-        for k in kwargs.keys():
-            proxy_info.update({k : kwargs[k]})
+        for k in list(kwargs.keys()):
+            proxy_info.update({k: kwargs[k]})
 
-        if proxy_info.has_key('user') and not proxy_info.has_key('pwd'):
-            proxy_info.update({'pwd' : getpass.getpass()})
-        proxy_support = urllib2.ProxyHandler({"http" :
-                    "http://%(user)s:%(pwd)s@%(host)s:%(port)d" % proxy_info})
-        opener = urllib2.build_opener(proxy_support, urllib2.HTTPHandler)
-        urllib2.install_opener(opener)
+        if 'user' in proxy_info and 'pwd' not in proxy_info:
+            proxy_info.update({'pwd': getpass.getpass()})
+        proxy_support = ProxyHandler(
+            {"http": "http://%(user)s:%(pwd)s@%(host)s:%(port)d" % proxy_info}
+        )
+        opener = build_opener(proxy_support, HTTPHandler)
+        install_opener(opener)
         del proxy_info, proxy_support
 
     def loadSBMLFileFromDisk(self, File, Dir=None):
@@ -143,12 +187,12 @@ class SBWLayoutWebLink(object):
             self.svg = None
             self.DRAWNETWORKLOADED = False
             self.LAYOUTMODULELOADED = False
-            sbmlF = file(path, 'r')
+            sbmlF = open(path, 'r')
             self.sbml = sbmlF.read()
             sbmlF.close()
             return True
         else:
-            print "%s is an invalid path" % path
+            print("%s is an invalid path" % path)
             return False
 
     def loadSBMLFromString(self, str):
@@ -160,69 +204,74 @@ class SBWLayoutWebLink(object):
         return True
 
     def urlGET(self, host, urlpath):
-        url = 'http://%s%s' % (host,urlpath)
-        con = urllib2.urlopen(url)
+        url = 'http://%s%s' % (host, urlpath)
+        con = urlopen(url)
         resp = con.read()
         if self.DEBUGMODE:
-            print con.headers
+            print(con.headers)
         if self.DEBUGMODE and self.DEBUGLEVEL == 2:
-            print resp
+            print(resp)
         con.close()
         return resp
 
     def urlPOST(self, host, urlpath, data):
         assert type(data) == dict, '\nData must be a dictionary'
         url = 'http://%s%s' % (host, urlpath)
-        con = urllib2.urlopen(url, urllib.urlencode(data))
+        con = urlopen(url, urlencode(data))
         resp = con.read()
         if self.DEBUGMODE:
-            print con.headers
+            print(con.headers)
         if self.DEBUGMODE and self.DEBUGLEVEL == 2:
-            print resp
+            print(resp)
         con.close()
         return resp
 
     def getVersion(self):
-        print 'Inspector.getVersion()'
+        print('Inspector.getVersion()')
         ver = self.urlGET(self.sbwhost, '/generate/Inspector.asmx/getVersion')
-        ver = ver.replace('<?xml version="1.0" encoding="utf-8"?>','')
-        ver = ver.replace('<string xmlns="http://www.sys-bio.org/">','')
-        ver = ver.replace('</string>','')
+        ver = ver.replace('<?xml version="1.0" encoding="utf-8"?>', '')
+        ver = ver.replace('<string xmlns="http://www.sys-bio.org/">', '')
+        ver = ver.replace('</string>', '')
         return ver
 
     def drawNetworkLoadSBML(self):
-        print 'DrawNetwork.loadSBML()'
+        print('DrawNetwork.loadSBML()')
         assert self.sbml != None, '\nNo SBML file loaded'
-        data = {'var0' : self.sbml}
+        data = {'var0': self.sbml}
         self.DRAWNETWORKLOADED = True
         return self.urlPOST(self.sbwhost, '/generate/DrawNetwork.asmx/loadSBML', data)
 
     def drawNetworkGetSBMLwithLayout(self):
-        print 'DrawNetwork.getSBML()'
+        print('DrawNetwork.getSBML()')
         assert self.DRAWNETWORKLOADED, '\nSBML not loaded into DrawNetwork module'
         sbml = self.urlGET(self.sbwhost, '/generate/DrawNetwork.asmx/getSBML')
-        sbml = sbml.replace('&gt;','>')
-        sbml = sbml.replace('&lt;','<')
-        sbml = sbml.replace('''<string xmlns="http://www.sys-bio.org/"><?xml version="1.0" encoding="utf-8"?>''','')
-        sbml = sbml.replace('</string>','')
+        sbml = sbml.replace('&gt;', '>')
+        sbml = sbml.replace('&lt;', '<')
+        sbml = sbml.replace(
+            '''<string xmlns="http://www.sys-bio.org/"><?xml version="1.0" encoding="utf-8"?>''',
+            '',
+        )
+        sbml = sbml.replace('</string>', '')
         self.sbmllayout = sbml
         return True
 
     def layoutModuleLoadSBML(self):
-        print 'SBMLLayoutModule.loadSBML()'
+        print('SBMLLayoutModule.loadSBML()')
         assert self.sbmllayout != None, '\nNo SBML Layout loaded'
-        data = {'var0' : self.sbmllayout}
+        data = {'var0': self.sbmllayout}
         self.LAYOUTMODULELOADED = True
-        return self.urlPOST(self.sbwhost, '/generate/SBMLLayoutModule.asmx/loadSBML', data)
+        return self.urlPOST(
+            self.sbwhost, '/generate/SBMLLayoutModule.asmx/loadSBML', data
+        )
 
     def layoutModuleGetSVG(self):
         assert self.LAYOUTMODULELOADED, '\nSBML not loaded into SBMLLayout module'
         svg = self.urlGET(self.sbwhost, '/generate/SBMLLayoutModule.asmx/getSVG')
-        svg = svg.replace('&gt;','>')
-        svg = svg.replace('&lt;','<')
-        svg = svg.replace('''<string xmlns="http://www.sys-bio.org/">''','')
-        svg = svg.replace('''<?xml version="1.0" encoding="utf-8"?>''','')
-        svg = svg.replace('</string>','')
+        svg = svg.replace('&gt;', '>')
+        svg = svg.replace('&lt;', '<')
+        svg = svg.replace('''<string xmlns="http://www.sys-bio.org/">''', '')
+        svg = svg.replace('''<?xml version="1.0" encoding="utf-8"?>''', '')
+        svg = svg.replace('</string>', '')
         self.svg = svg
         return True
 
@@ -247,7 +296,7 @@ class METATOOLlink(object):
     __emode_userout__ = 0
     __emode_file__ = None
     __metatool_file__ = None
-    #EModes = ''
+    # EModes = ''
 
     def __init__(self, mod, __metatool_path__=None):
         # Initialise elementary modes
@@ -261,23 +310,31 @@ class METATOOLlink(object):
         self.__emode_file__ = self.__mod__.ModelFile[:-4] + '_emodes'
         self.__metatool_file__ = self.__mod__.ModelFile[:-4] + '_metatool'
         if os.sys.platform == 'win32':
-            self.__emode_exe_int__ = os.path.join(self.__metatool_path__,'meta43_int.exe')
-            self.__emode_exe_dbl__ = os.path.join(self.__metatool_path__,'meta43_double.exe')
+            self.__emode_exe_int__ = os.path.join(
+                self.__metatool_path__, 'meta43_int.exe'
+            )
+            self.__emode_exe_dbl__ = os.path.join(
+                self.__metatool_path__, 'meta43_double.exe'
+            )
         else:
-            self.__emode_exe_int__ = os.path.join(self.__metatool_path__,'meta43_int')
-            self.__emode_exe_dbl__ = os.path.join(self.__metatool_path__,'meta43_double')
+            self.__emode_exe_int__ = os.path.join(self.__metatool_path__, 'meta43_int')
+            self.__emode_exe_dbl__ = os.path.join(
+                self.__metatool_path__, 'meta43_double'
+            )
 
         if os.path.exists(self.__emode_exe_int__):
-            print 'Using METATOOL int',
+            print('Using METATOOL int', end=' ')
             self.__emode_intmode__ = True
         else:
             self.__emode_exe_int__ = None
         if os.path.exists(self.__emode_exe_dbl__):
-            print '\b\b\b\bdbl'
+            print('\b\b\b\bdbl')
             self.__emode_intmode__ = False
         else:
             self.__emode_exe_dbl__ = None
-        assert self.__emode_exe_dbl__ != None or self.__emode_exe_int__ != None, "\nMETATOOL binaries not available"
+        assert (
+            self.__emode_exe_dbl__ != None or self.__emode_exe_int__ != None
+        ), "\nMETATOOL binaries not available"
 
     def doEModes(self):
         """
@@ -293,9 +350,15 @@ class METATOOLlink(object):
         None
 
         """
-        print 'METATOOL is a C program developed from 1998 to 2000 by Thomas Pfeiffer (Berlin)'
-        print 'in cooperation with Stefan Schuster and Ferdinand Moldenhauer (Berlin) and Juan Carlos Nuno (Madrid).'
-        print 'http://www.biologie.hu-berlin.de/biophysics/Theory/tpfeiffer/metatool.html'
+        print(
+            'METATOOL is a C program developed from 1998 to 2000 by Thomas Pfeiffer (Berlin)'
+        )
+        print(
+            'in cooperation with Stefan Schuster and Ferdinand Moldenhauer (Berlin) and Juan Carlos Nuno (Madrid).'
+        )
+        print(
+            'http://www.biologie.hu-berlin.de/biophysics/Theory/tpfeiffer/metatool.html'
+        )
 
         goMode = 0
         fileIn = 'pysces_metatool.dat'
@@ -304,14 +367,14 @@ class METATOOLlink(object):
         goMode = 1
         if goMode == 1:
             # Build MetaTool input file
-            File = open(os.path.join(self.__mod__.ModelOutput,fileIn),'w')
+            File = open(os.path.join(self.__mod__.ModelOutput, fileIn), 'w')
             # Determine type of reaction
             out1 = []
             for key in self.__mod__.__nDict__:
-                #print key
-                #print self.__mod__.__nDict__[key]['Type']
-                out1.append((key,self.__mod__.__nDict__[key]['Type']))
-            #print '\nExtracting metatool information from network dictionary ...\n'
+                # print key
+                # print self.__mod__.__nDict__[key]['Type']
+                out1.append((key, self.__mod__.__nDict__[key]['Type']))
+            # print '\nExtracting metatool information from network dictionary ...\n'
             File.write('-ENZREV\n')
             for x in out1:
                 if x[1] == 'Rever':
@@ -337,37 +400,37 @@ class METATOOLlink(object):
                 reList = self.__mod__.__nDict__[x]['Reagents']
                 subs = ''
                 prods = ''
-                #print 'Reaction: ' + x
+                # print 'Reaction: ' + x
                 for y in reList:
-                    if self.__emode_intmode__ == 1: # use int elementary modes
-                        if abs(int(reList[y]))/abs(float(reList[y])) != 1.0:
-                            print 'INFO: Coefficient not integer = ' + `reList[y]`
+                    if self.__emode_intmode__ == 1:  # use int elementary modes
+                        if abs(int(reList[y])) / abs(float(reList[y])) != 1.0:
+                            print('INFO: Coefficient not integer = ' + repr(reList[y]))
                             allInt = 0
                         if reList[y] < 0:
-                            #print y.replace('self.','') + ' : substrate'
+                            # print y.replace('self.','') + ' : substrate'
                             if abs(int(reList[y])) != 1:
-                                subs += `abs(int(reList[y]))` + ' '
-                            subs += y.replace('self.','')
+                                subs += repr(abs(int(reList[y]))) + ' '
+                            subs += y.replace('self.', '')
                             subs += ' + '
                         else:
-                            #print y.replace('self.','') + ' : product '
+                            # print y.replace('self.','') + ' : product '
                             if abs(int(reList[y])) != 1:
-                                prods += `abs(int(reList[y]))` + ' '
-                            prods += y.replace('self.','')
+                                prods += repr(abs(int(reList[y]))) + ' '
+                            prods += y.replace('self.', '')
                             prods += ' + '
-                        #output.append(x + ' : ' + subs[:-3] + ' = ' + prods[:-3] + ' .')
-                    else: # use float/double elementary mode
+                        # output.append(x + ' : ' + subs[:-3] + ' = ' + prods[:-3] + ' .')
+                    else:  # use float/double elementary mode
                         if reList[y] < 0.0:
-                            #print y.replace('self.','') + ' : substrate'
+                            # print y.replace('self.','') + ' : substrate'
                             if abs(float(reList[y])) != 1.0:
-                                subs += `abs(float(reList[y]))` + ' '
-                            subs += y.replace('self.','')
+                                subs += repr(abs(float(reList[y]))) + ' '
+                            subs += y.replace('self.', '')
                             subs += ' + '
                         else:
-                            #print y.replace('self.','') + ' : product '
+                            # print y.replace('self.','') + ' : product '
                             if abs(float(reList[y])) != 1.0:
-                                prods += `abs(float(reList[y]))` + ' '
-                            prods += y.replace('self.','')
+                                prods += repr(abs(float(reList[y]))) + ' '
+                            prods += y.replace('self.', '')
                             prods += ' + '
                 output.append(x + ' : ' + subs[:-3] + ' = ' + prods[:-3] + ' .')
 
@@ -385,27 +448,33 @@ class METATOOLlink(object):
                 else:
                     eModeExe = self.__emode_exe_dbl__
 
-                print '\nMetatool running ...\n'
+                print('\nMetatool running ...\n')
                 ######### UPDATE:
                 # Actually works fine on windows and posix - johann 20081128
-                print 'Generic run'
-                os.spawnl(os.P_WAIT, eModeExe, eModeExe, os.path.join(self.__mod__.ModelOutput,fileIn), os.path.join(self.__mod__.ModelOutput,fileOut))
-                print '\nMetatool analysis complete\n'
+                print('Generic run')
+                os.spawnl(
+                    os.P_WAIT,
+                    eModeExe,
+                    eModeExe,
+                    os.path.join(self.__mod__.ModelOutput, fileIn),
+                    os.path.join(self.__mod__.ModelOutput, fileOut),
+                )
+                print('\nMetatool analysis complete\n')
 
                 # Parse MetaTool output file and store the result in a string
                 go = 0
-                go2  = 0
+                go2 = 0
                 result = ''
                 end = ''
 
                 try:
-                    file2 = open(os.path.join(self.__mod__.ModelOutput,fileOut), 'r')
+                    file2 = open(os.path.join(self.__mod__.ModelOutput, fileOut), 'r')
                     for line in file2:
-                        c = re.match('ELEMENTARY MODES',line)
-                        d = re.match(' enzymes',line)
-                        e = re.match('The elementary mode',line)
-                        f = re.match('\n',line)
-                        g = re.match('The elementary',line)
+                        c = re.match('ELEMENTARY MODES', line)
+                        d = re.match(' enzymes', line)
+                        e = re.match('The elementary mode', line)
+                        f = re.match('\n', line)
+                        g = re.match('The elementary', line)
                         if c != None:
                             go = 1
                             go2 = 0
@@ -414,12 +483,12 @@ class METATOOLlink(object):
                         if e != None:
                             go2 = 0
                         if go == 1 and go2 == 1 and f == None:
-                            line = line.replace('reversible','\n  reversible\n')
-                            line = line.replace('ir\n  ','\n  ir')
+                            line = line.replace('reversible', '\n  reversible\n')
+                            line = line.replace('ir\n  ', '\n  ir')
                             if self.__emode_intmode__ == 1:
-                                line = line.replace('] ',']\n ')
+                                line = line.replace('] ', ']\n ')
                             else:
-                                line = line.replace(') ',')\n ',1)
+                                line = line.replace(') ', ')\n ', 1)
                             result += line
                         if go == 1 and g != None:
                             end += line
@@ -429,34 +498,56 @@ class METATOOLlink(object):
                     file2.close()
 
                     if self.__emode_userout__ == 1:
-                        fileo = open(os.path.join(self.__mod__.ModelOutput,self.__metatool_file__) + '.in','w')
-                        filer = open(os.path.join(self.__mod__.ModelOutput,fileIn),'r')
+                        fileo = open(
+                            os.path.join(
+                                self.__mod__.ModelOutput, self.__metatool_file__
+                            )
+                            + '.in',
+                            'w',
+                        )
+                        filer = open(
+                            os.path.join(self.__mod__.ModelOutput, fileIn), 'r'
+                        )
                         for line in filer:
                             fileo.write(line)
                         fileo.write('\n\n')
                         filer.close()
                         fileo.close()
-                        filer = open(os.path.join(self.__mod__.ModelOutput,fileOut),'r')
-                        fileo = open(os.path.join(self.__mod__.ModelOutput,self.__metatool_file__) + '.out','w')
+                        filer = open(
+                            os.path.join(self.__mod__.ModelOutput, fileOut), 'r'
+                        )
+                        fileo = open(
+                            os.path.join(
+                                self.__mod__.ModelOutput, self.__metatool_file__
+                            )
+                            + '.out',
+                            'w',
+                        )
                         for line in filer:
                             fileo.write(line)
                         filer.close()
                         fileo.close()
-                    os.remove(os.path.join(self.__mod__.ModelOutput,fileIn))
-                    os.remove(os.path.join(self.__mod__.ModelOutput,fileOut))
-                except Exception, EX:
-                    print 'doEmode:', EX
-                    print 'WARNING: Unable to open MetaTool output file\nPlease check the MetaTool executables: '
+                    os.remove(os.path.join(self.__mod__.ModelOutput, fileIn))
+                    os.remove(os.path.join(self.__mod__.ModelOutput, fileOut))
+                except Exception as EX:
+                    print('doEmode:', EX)
+                    print(
+                        'WARNING: Unable to open MetaTool output file\nPlease check the MetaTool executables: '
+                    )
                     if os.name == 'posix':
-                        print '/MetaTool/meta43_double /MetaTool/meta43_int\nand their permissions'
+                        print(
+                            '/MetaTool/meta43_double /MetaTool/meta43_int\nand their permissions'
+                        )
                     else:
-                        print '/MetaTool/meta43_double.exe /MetaTool/meta43_int.exe'
+                        print('/MetaTool/meta43_double.exe /MetaTool/meta43_int.exe')
             else:
-                print '\nINFO: non-integer coefficients\
+                print(
+                    '\nINFO: non-integer coefficients\
                 \nTry using the double eMode function: self.__emode_intmode__=0'
+                )
                 result = 'Elementary modes not calculated\n'
         else:
-            print '\nNo elementary mode calculation possible - no meta43_xxx.exe'
+            print('\nNo elementary mode calculation possible - no meta43_xxx.exe')
             result = 'Elementary modes not calculated\n'
         self.EModes = result
 
@@ -469,13 +560,17 @@ class METATOOLlink(object):
         """
         try:
             a = self.EModes
-            FF = cStringIO.StringIO()
+            FF = io.StringIO()
             FF.write(self.EModes)
             FF.reset()
             output = []
             for line in FF:
-                if re.match('  ',line) and not re.match('  reversible',line) and not re.match('  irreversible',line):
-                    tmp = [el for el in line.replace('\n','').split(' ') if el != '']
+                if (
+                    re.match('  ', line)
+                    and not re.match('  reversible', line)
+                    and not re.match('  irreversible', line)
+                ):
+                    tmp = [el for el in line.replace('\n', '').split(' ') if el != '']
                     tmpOut = []
                     skip = False
                     for el in range(len(tmp)):
@@ -484,15 +579,15 @@ class METATOOLlink(object):
                         elif tmp[el][0] != '(':
                             tmpOut.append(tmp[el])
                         elif tmp[el][0] == '(':
-                            tmpOut.append(tmp[el]+')'+tmp[el+1][:-1])
+                            tmpOut.append(tmp[el] + ')' + tmp[el + 1][:-1])
                             skip = True
                     output.append(tmpOut)
             return output
-        except AttributeError, atx:
-            print atx
-            print '\nINFO: Please run doEModes() first\n'
+        except AttributeError as atx:
+            print(atx)
+            print('\nINFO: Please run doEModes() first\n')
 
-    def showEModes(self,File=None):
+    def showEModes(self, File=None):
         """
         showEModes(File=None)
 
@@ -505,25 +600,23 @@ class METATOOLlink(object):
         """
         try:
             if File != None:
-                #assert type(File) == file, 'showEmodes() needs an open file object'
-                print '\nElementary modes written to file\n'
-                f = open(os.path.join(self.__mod__.ModelOutput,self.__emode_file__ + '.out'),'w')
+                # assert type(File) == file, 'showEmodes() needs an open file object'
+                print('\nElementary modes written to file\n')
+                f = open(
+                    os.path.join(
+                        self.__mod__.ModelOutput, self.__emode_file__ + '.out'
+                    ),
+                    'w',
+                )
                 f.write('\n## Elementary modes\n')
                 f.write(self.EModes)
                 f.close()
             else:
-                print '\nElementary modes\n'
-                print self.EModes
-        except AttributeError, atx:
-            print atx
-            print '\nINFO: Please run doEModes() first\n'
-
-
-
-
-
-
-#stochsim link
+                print('\nElementary modes\n')
+                print(self.EModes)
+        except AttributeError as atx:
+            print(atx)
+            print('\nINFO: Please run doEModes() first\n')
 
 
 '''
@@ -690,9 +783,9 @@ class StomPyInterface(object):
         ##  pscmod.data_stochsim = IntegrationStochasticDataObj()
         ##  pscmod.data_stochsim.setTime(sim_dat[:,0])
         ##  pscmod.data_stochsim.setSpecies(sim_dat[:,1:-1], self.SSA_SPECIES)
-        
+
         pscmod.data_stochsim = self.SSA.data_stochsim
-        
+
         if self.STP_WAITING_TIMES:
             wtimes, wt_lbls = self.getWaitingtimesData(reactions=None,lbls=True)
             pscmod.data_stochsim.setWaitingtimes(wtimes, wt_lbls)
@@ -743,14 +836,14 @@ class StomPyInterface(object):
         ##  pscmod.data_stochsim = IntegrationStochasticDataObj()
         ##  pscmod.data_stochsim.setTime(sim_dat[:,0])
         ##  pscmod.data_stochsim.setSpecies(sim_dat[:,1:-1], self.SSA_SPECIES)
-        
+
         pscmod.data_stochsim = self.SSA.data_stochsim
-        
+
         if self.STP_WAITING_TIMES:
             wtimes, wt_lbls = self.getWaitingtimesData(reactions=None,lbls=True)
             pscmod.data_stochsim.setWaitingtimes(wtimes, wt_lbls)
         if self.STP_TRACK_PROPENSITIES:
-            pscmod.data_stochsim.setPropensities(self.SSA.SSA.propensities_output)            
+            pscmod.data_stochsim.setPropensities(self.SSA.SSA.propensities_output)
         pscmod.data_stochsim.TYPE_INFO = 'Stochastic'
 
 
@@ -871,33 +964,33 @@ class IntegrationStochasticDataObj(object):
     def setWaitingtimes(self, waiting_times, lbls=None):
         """
         Set the `waiting_times` this data structure is not an array but a nested list of: waiting time log bins per reaction per trajectory::
-         
+
         waiting_times = [traj_1, ..., traj_n]
         traj_1 = [wt_J1, ..., wt_Jn] # in order of SSA_REACTIONS
         wt_J1 = (xval, yval, nbin)
         xval =[x_1, ..., x_n]
         yval =[y_1, ..., y_n]
         nbin = n
-        
+
          - *waiting_times* a list of waiting times
          - *lbls* [default=None] a list of matching reaction names
-         
+
         """
         self.waiting_times = waiting_times
         self.HAS_WAITING_TIMES = True
         if lbls != None:
             self.waiting_time_labels = lbls
-            
+
 
     def setPropensities(self, propensities, lbls=None):
         """
         Sets an array of propensities.
-        
+
          - *propensities* a list of propensities
-         - *lbls* [default=None] a list of matching reaction names       
-         
+         - *lbls* [default=None] a list of matching reaction names
+
         """
-        
+
         if lbls == None:
             LB = copy.copy(propensities[0])
             lbls = LB[1:]
@@ -912,7 +1005,7 @@ class IntegrationStochasticDataObj(object):
             self.propensities_labels = lbls
         ##  print self.propensities_labels
         ##  print self.propensities
-            
+
 
     def setXData(self, xdata, lbls=None):
         """
@@ -992,9 +1085,9 @@ class IntegrationStochasticDataObj(object):
     def getPropensities(self, lbls=False):
         """
         Return time+propensity array
-        
+
          - *lbls* [default=False] return only the time+propensity array or optionally both the data array and a list of column label
-        
+
         """
         #assert self.propensities != None, "\nNo propensities"
         output = None
@@ -1194,7 +1287,7 @@ class PysMod:
         elif plot == 'waiting_times':
             dataLst, labels = self.data_stochsim.getWaitingtimes(lbls=True)
             format='points'
-            ##  data, labels = self.data_stochsim.getRates(lbls=True)            
+            ##  data, labels = self.data_stochsim.getRates(lbls=True)
         else:
             plot = [at for at in plot if at in self.__species__+[self.data_stochsim.time_label]+self.data_stochsim.propensities_labels]
             kwargs = {'lbls' : True}
@@ -1202,9 +1295,9 @@ class PysMod:
             if len(plot) > 0:
                 data, labels = self.data_stochsim.getSimData(*plot, **kwargs)
         del allowedplots
-        
+
         xu = 'Time (%(multiplier)s x %(kind)s x 10**%(scale)s)**%(exponent)s' % self.__uDict__['time']
-        
+
         if plot == 'waiting_times':
             xu = 'Inter-arrival time (%s)' % xu
 
@@ -1216,7 +1309,7 @@ class PysMod:
                 for d in range(len(dataLst[wt])):
                     D = dataLst[wt][d]
                     if plt.__USE_MATPLOTLIB__ and d > 0:
-                        plt.m.hold(True)                         
+                        plt.m.hold(True)
                     if D != None and len(D[0]) > 0 and len(D[1]) > 0:
                         data = numpy.vstack([D[0], D[1]]).transpose()
                         if min(D[0]) < xrng_start and min(D[0]) > 0.0:
@@ -1226,7 +1319,7 @@ class PysMod:
                         if min(D[1]) < yrng_start and min(D[1]) > 0.0:
                             yrng_start = min(D[1])
                         if max(D[1]) > yrng_end:
-                            yrng_end = max(D[1])                    
+                            yrng_end = max(D[1])
                         plt.plotLines(data, 0, [1], titles=['Time']+[labels[d]], formats=[format])
             plt.setRange('x', xrng_start*0.8, xrng_end*1.2)
             plt.setRange('y', yrng_start*0.8, yrng_end*1.2)
@@ -1241,7 +1334,7 @@ class PysMod:
             end = RngTime[-1] + 0.2*RngTime[-1]
             plt.setRange('x', RngTime[0], end)
             del RngTime
-        
+
         # For now StochPy results are plotted as Amounts directly from StochPy
         M = 'Amount'
         ##  if self.__KeyWords__['Output_In_Conc']:
@@ -1349,7 +1442,7 @@ class PysMod:
         - *fmt* [default='lines'] use UPI or backend specific keys
 
         """
-        
+
         self.doStochSim(end=end, mode=mode, method=method,trajectories=1)
         self.StochSimPlot(plot='species', filename=filename, log=log, format=fmt)
 
@@ -1363,15 +1456,3 @@ if not _HAVE_STOMPY:
     PysMod.StochSimPlot = nofunc
 
 '''
-
-
-
-
-
-
-
-
-
-
-
-
