@@ -1,7 +1,7 @@
 """
 PySCeS - Python Simulator for Cellular Systems (http://pysces.sourceforge.net)
 
-Copyright (C) 2004-2020 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
+Copyright (C) 2004-2022 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
 
 Brett G. Olivier (bgoli@users.sourceforge.net)
 Triple-J Group for Molecular Cell Physiology
@@ -32,13 +32,16 @@ import os, copy, time
 import numpy
 from pysces import PyscesStoich
 from pysces import PyscesParse
+
 mach_spec = numpy.MachAr()
 pscParser = PyscesParse.PySCeSParser(debug=0)
 
+
 class PyscesInputFileParser(object):
     """
     This class contains the PySCeS model loading and Stoichiometric Analysis methods
     """
+
     ModelDir = None
     ModelFile = None
     ModelOutput = None
@@ -54,9 +57,13 @@ class PyscesInputFileParser(object):
             assert os.path.exists(output_dir), "\n%s is not a valid path" % output_dir
         self.__settings__ = {}
         # Initialize stoichiometric precision
-        self.__settings__['stoichiometric_analysis_fp_zero'] = mach_spec.eps*2.0e4
-        self.__settings__['stoichiometric_analysis_lu_precision'] = self.__settings__['stoichiometric_analysis_fp_zero']
-        self.__settings__['stoichiometric_analysis_gj_precision'] = self.__settings__['stoichiometric_analysis_lu_precision']*10.0
+        self.__settings__['stoichiometric_analysis_fp_zero'] = mach_spec.eps * 2.0e4
+        self.__settings__['stoichiometric_analysis_lu_precision'] = self.__settings__[
+            'stoichiometric_analysis_fp_zero'
+        ]
+        self.__settings__['stoichiometric_analysis_gj_precision'] = (
+            self.__settings__['stoichiometric_analysis_lu_precision'] * 10.0
+        )
         self.__settings__['enable_deprecated_attr'] = False
         self.InitialiseInputFile()
         self.N = self.buildN()
@@ -71,14 +78,20 @@ class PyscesInputFileParser(object):
         None
 
         """
-        self.__parseOK = 1 # check that model has parsed ok?
+        self.__parseOK = 1  # check that model has parsed ok?
         try:
-            if os.path.exists(os.path.join(self.ModelDir,self.ModelFile)):
+            if os.path.exists(os.path.join(self.ModelDir, self.ModelFile)):
                 pass
             else:
-                print('\nInvalid self.ModelFile: ' + os.path.join(self.ModelDir,self.ModelFile))
+                print(
+                    '\nInvalid self.ModelFile: '
+                    + os.path.join(self.ModelDir, self.ModelFile)
+                )
         except:
-            print('WARNING: Problem verifying: ' + os.path.join(self.ModelDir,self.ModelFile))
+            print(
+                'WARNING: Problem verifying: '
+                + os.path.join(self.ModelDir, self.ModelFile)
+            )
 
         if self.ModelFile[-4:] == '.psc':
             pass
@@ -88,19 +101,21 @@ class PyscesInputFileParser(object):
 
         print('\nParsing file: %s' % os.path.join(self.ModelDir, self.ModelFile))
 
-        pscParser.ParsePSC(self.ModelFile,self.ModelDir,self.ModelOutput)
+        pscParser.ParsePSC(self.ModelFile, self.ModelDir, self.ModelOutput)
         print(' ')
 
         badlist = pscParser.KeywordCheck(pscParser.ReactionIDs)
-        badlist = pscParser.KeywordCheck(pscParser.Inits,badlist)
+        badlist = pscParser.KeywordCheck(pscParser.Inits, badlist)
 
         if len(badlist) != 0:
-            print('\n******************************\nPSC input file contains PySCeS keywords please rename them and reload:')
+            print(
+                '\n******************************\nPSC input file contains PySCeS keywords please rename them and reload:'
+            )
             for item in badlist:
                 print('   --> ' + item)
             print('******************************\n')
             self.__parseOK = 0
-            #assert len(badlist) != 0, 'Keyword error, please check input file'
+            # assert len(badlist) != 0, 'Keyword error, please check input file'
 
         if self.__parseOK:
             # brett 2008
@@ -114,20 +129,28 @@ class PyscesInputFileParser(object):
             # set parameters and add to __InitDict__
             for p in list(self.__pDict__.keys()):
                 setattr(self, self.__pDict__[p]['name'], self.__pDict__[p]['initial'])
-                self.__InitDict__.update({self.__pDict__[p]['name'] : self.__pDict__[p]['initial']})
+                self.__InitDict__.update(
+                    {self.__pDict__[p]['name']: self.__pDict__[p]['initial']}
+                )
             # set species and add to __InitDict__ and set mod.Xi_init
             for s in list(self.__sDict__.keys()):
                 setattr(self, self.__sDict__[s]['name'], self.__sDict__[s]['initial'])
                 if not self.__sDict__[s]['fixed']:
-                    setattr(self, self.__sDict__[s]['name']+'_init', self.__sDict__[s]['initial'])
-                self.__InitDict__.update({self.__sDict__[s]['name'] : self.__sDict__[s]['initial']})
+                    setattr(
+                        self,
+                        self.__sDict__[s]['name'] + '_init',
+                        self.__sDict__[s]['initial'],
+                    )
+                self.__InitDict__.update(
+                    {self.__sDict__[s]['name']: self.__sDict__[s]['initial']}
+                )
 
             # setup keywords
             self.__KeyWords__ = pscParser.KeyWords.copy()
             if self.__KeyWords__['Modelname'] == None:
-                self.__KeyWords__['Modelname'] = self.ModelFile.replace('.psc','')
+                self.__KeyWords__['Modelname'] = self.ModelFile.replace('.psc', '')
             if self.__KeyWords__['Description'] == None:
-                self.__KeyWords__['Description'] = self.ModelFile.replace('.psc','')
+                self.__KeyWords__['Description'] = self.ModelFile.replace('.psc', '')
             # if SpeciesTypes undefined assume []
             if self.__KeyWords__['Species_In_Conc'] == None:
                 self.__KeyWords__['Species_In_Conc'] = True
@@ -177,17 +200,28 @@ class PyscesInputFileParser(object):
             self.__eDict__ = pscParser.Events.copy()
 
             ##  if pscParser.ModelUsesNumpyFuncs:
-                ##  print 'Numpy functions detected in kinetic laws.\n'
+            ##  print 'Numpy functions detected in kinetic laws.\n'
         else:
             print('\nERROR: model parsing error, please check input file.\n')
         # added in a check for model correctness and human error reporting (1=ok, 0=error)
         if len(pscParser.SymbolErrors) != 0:
             print('\nUndefined symbols:\n%s' % self.SymbolErrors)
         if not pscParser.ParseOK:
-            print('\n\n*****\nModel parsing errors detected in input file '+ self.ModelFile +'\n*****')
+            print(
+                '\n\n*****\nModel parsing errors detected in input file '
+                + self.ModelFile
+                + '\n*****'
+            )
             print('\nInput file errors')
             for error in pscParser.LexErrors:
-                print(error[0] + 'in line:\t' + str(error[1]) + ' ('+ error[2][:20] +' ...)')
+                print(
+                    error[0]
+                    + 'in line:\t'
+                    + str(error[1])
+                    + ' ('
+                    + error[2][:20]
+                    + ' ...)'
+                )
             print('\nParser errors')
             for error in pscParser.ParseErrors:
                 try:
@@ -207,15 +241,17 @@ class PyscesInputFileParser(object):
         None
 
         """
-        VarReagents = ['self.'+s for s in self.__species__]
-        StoicMatrix = numpy.zeros((len(VarReagents),len(self.__reactions__)),'d')
+        VarReagents = ['self.' + s for s in self.__species__]
+        StoicMatrix = numpy.zeros((len(VarReagents), len(self.__reactions__)), 'd')
         for reag in VarReagents:
             for id in self.__reactions__:
                 if reag in list(self.__nDict__[id]['Reagents'].keys()):
-                    StoicMatrix[VarReagents.index(reag)][self.__reactions__.index(id)] = self.__nDict__[id]['Reagents'][reag]
+                    StoicMatrix[VarReagents.index(reag)][
+                        self.__reactions__.index(id)
+                    ] = self.__nDict__[id]['Reagents'][reag]
         return StoicMatrix
 
-    def Stoichiometry_Init(self,nmatrix):
+    def Stoichiometry_Init(self, nmatrix):
         """
         Stoichiometry_Init(nmatrix,load=0)
 
@@ -229,10 +265,10 @@ class PyscesInputFileParser(object):
         load [default=0]: try to load a saved stoichiometry (1)
 
         """
-        #print 'Instantiating new stoichiometry ...'
+        # print 'Instantiating new stoichiometry ...'
         stc = PyscesStoich.Stoich(nmatrix)
         status = 0
-        return stc,status
+        return stc, status
 
     def Stoichiometry_Analyse(self):
         override = 0
@@ -253,16 +289,19 @@ class PyscesInputFileParser(object):
 
         """
         if not override:
-            self.nmatrix = self.buildN()            #Creates the model N
-            #print '\nintializing N\n'
+            self.nmatrix = self.buildN()  # Creates the model N
+            # print '\nintializing N\n'
         else:
             print('\nStoichiometric override active\n')
 
-        assert len(self.nmatrix) > 0, '\nUnable to generate Stoichiometric Matrix! model has:\n%s reactions\n%s species\nwhat did you have in mind?\n' % (len(self.__reactions__), len(self.__species__))
+        assert len(self.nmatrix) > 0, (
+            '\nUnable to generate Stoichiometric Matrix! model has:\n%s reactions\n%s species\nwhat did you have in mind?\n'
+            % (len(self.__reactions__), len(self.__species__))
+        )
 
         ##  self.__nmatrix__ = copy.copy(self.nmatrix)
-        self.__nmatrix__ = self.nmatrix # done with caution brett2008
-        self.__Nshape__ = self.nmatrix.shape               #Get the shape of N
+        self.__nmatrix__ = self.nmatrix  # done with caution brett2008
+        self.__Nshape__ = self.nmatrix.shape  # Get the shape of N
         ##  self.__Vtemp__ = numpy.zeros((self.__Nshape__[1])) # going going ....
 
         # get stoich instance and whether it was analysed or loaded - brett 20050830
@@ -271,40 +310,80 @@ class PyscesInputFileParser(object):
         # if not loaded analyze - brett 20050830
         if not stc_load:
             # technically this means we can define this on the fly - brett #20051013
-            self.__structural__.stoichiometric_analysis_fp_zero = self.__settings__['stoichiometric_analysis_fp_zero']
-            self.__structural__.stoichiometric_analysis_lu_precision = self.__settings__['stoichiometric_analysis_lu_precision']
-            self.__structural__.stoichiometric_analysis_gj_precision = self.__settings__['stoichiometric_analysis_gj_precision']
-            self.__structural__.AnalyseL()           #Get all L related stuff
-            self.__structural__.AnalyseK()           #Get all K related stuff
+            self.__structural__.stoichiometric_analysis_fp_zero = self.__settings__[
+                'stoichiometric_analysis_fp_zero'
+            ]
+            self.__structural__.stoichiometric_analysis_lu_precision = self.__settings__[
+                'stoichiometric_analysis_lu_precision'
+            ]
+            self.__structural__.stoichiometric_analysis_gj_precision = self.__settings__[
+                'stoichiometric_analysis_gj_precision'
+            ]
+            self.__structural__.AnalyseL()  # Get all L related stuff
+            self.__structural__.AnalyseK()  # Get all K related stuff
 
-
-        #test matrix values against __settings__['stoichiometric_analysis_lu_precision']
-        lsmall,lbig = self.__structural__.MatrixValueCompare(self.__structural__.lzeromatrix)
-        ksmall,kbig = self.__structural__.MatrixValueCompare(self.__structural__.kzeromatrix)
+        # test matrix values against __settings__['stoichiometric_analysis_lu_precision']
+        lsmall, lbig = self.__structural__.MatrixValueCompare(
+            self.__structural__.lzeromatrix
+        )
+        ksmall, kbig = self.__structural__.MatrixValueCompare(
+            self.__structural__.kzeromatrix
+        )
         SmallValueError = 0
-        if abs(lsmall) < self.__structural__.stoichiometric_analysis_lu_precision*10.0:
-            print('\nWARNING: values in L0matrix are close to stoichiometric precision!')
-            print('Stoichiometric LU precision:', self.__structural__.stoichiometric_analysis_lu_precision)
+        if (
+            abs(lsmall)
+            < self.__structural__.stoichiometric_analysis_lu_precision * 10.0
+        ):
+            print(
+                '\nWARNING: values in L0matrix are close to stoichiometric precision!'
+            )
+            print(
+                'Stoichiometric LU precision:',
+                self.__structural__.stoichiometric_analysis_lu_precision,
+            )
             print('L0 smallest abs(value)', abs(lsmall))
             print('Machine precision:', mach_spec.eps)
             SmallValueError = 1
-        if abs(ksmall) < self.__structural__.stoichiometric_analysis_lu_precision*10.0:
-            print('\nWARNING: values in K0matrix are close to stoichiometric precision!')
-            print('Stoichiometric precision:', self.__structural__.stoichiometric_analysis_lu_precision)
+        if (
+            abs(ksmall)
+            < self.__structural__.stoichiometric_analysis_lu_precision * 10.0
+        ):
+            print(
+                '\nWARNING: values in K0matrix are close to stoichiometric precision!'
+            )
+            print(
+                'Stoichiometric precision:',
+                self.__structural__.stoichiometric_analysis_lu_precision,
+            )
             print('K0 smallest abs(value)', abs(ksmall))
             print('Machine precision:', mach_spec.eps)
             SmallValueError = 1
         if SmallValueError:
-            input('\nStructural Analysis results may not be reliable!!!.\n\nTry change <mod>.__settings__["stoichiometric_analysis_lu_precision"] (see reference manual for details)\n\n\t press any key to continue: ')
+            input(
+                '\nStructural Analysis results may not be reliable!!!.\n\nTry change <mod>.__settings__["stoichiometric_analysis_lu_precision"] (see reference manual for details)\n\n\t press any key to continue: '
+            )
 
         # cross check that rank is consistant between K0 and L0
-        if self.__structural__.kzeromatrix.shape[0] != self.__structural__.lzeromatrix.shape[1]:
-            print('\nWARNING: the rank calculated by the Kand L analysis methods are not the same!')
-            print('\tK analysis calculates the rank as: ' + repr(self.__structural__.kzeromatrix.shape[0]))
-            print('\tL analysis calculates the rank as: ' + repr(self.__structural__.lzeromatrix.shape[1]))
+        if (
+            self.__structural__.kzeromatrix.shape[0]
+            != self.__structural__.lzeromatrix.shape[1]
+        ):
+            print(
+                '\nWARNING: the rank calculated by the Kand L analysis methods are not the same!'
+            )
+            print(
+                '\tK analysis calculates the rank as: '
+                + repr(self.__structural__.kzeromatrix.shape[0])
+            )
+            print(
+                '\tL analysis calculates the rank as: '
+                + repr(self.__structural__.lzeromatrix.shape[1])
+            )
             print('This is not good! Structural Analysis results are not reliable!!!\n')
-            assert self.__structural__.kzeromatrix.shape[0] == self.__structural__.lzeromatrix.shape[1], '\nStructuralAnalysis Error: rank mismatch'
-
+            assert (
+                self.__structural__.kzeromatrix.shape[0]
+                == self.__structural__.lzeromatrix.shape[1]
+            ), '\nStructuralAnalysis Error: rank mismatch'
 
         self.__HAS_FLUX_CONSERVATION__ = self.__structural__.info_flux_conserve
         self.__HAS_MOIETY_CONSERVATION__ = self.__structural__.info_moiety_conserve
@@ -320,7 +399,6 @@ class PyscesInputFileParser(object):
             self.kzeromatrix_row = self.__structural__.kzeromatrix_row
             self.kzeromatrix_col = self.__structural__.kzeromatrix_col
 
-
             self.lmatrix = self.__structural__.lmatrix
             self.lmatrix_row = self.__structural__.lmatrix_row
             self.lmatrix_col = self.__structural__.lmatrix_col
@@ -343,32 +421,60 @@ class PyscesInputFileParser(object):
 
         self.__structural__.species = self.species
         self.__structural__.reactions = self.reactions
-        self.Nmatrix = PyscesStoich.StructMatrix(self.__structural__.nmatrix, self.__structural__.nmatrix_row, self.__structural__.nmatrix_col)
+        self.Nmatrix = PyscesStoich.StructMatrix(
+            self.__structural__.nmatrix,
+            self.__structural__.nmatrix_row,
+            self.__structural__.nmatrix_col,
+        )
         self.Nmatrix.setRow(self.species)
         self.Nmatrix.setCol(self.reactions)
 
-        self.Nrmatrix = PyscesStoich.StructMatrix(self.__structural__.nrmatrix, self.__structural__.nrmatrix_row, self.__structural__.nrmatrix_col)
+        self.Nrmatrix = PyscesStoich.StructMatrix(
+            self.__structural__.nrmatrix,
+            self.__structural__.nrmatrix_row,
+            self.__structural__.nrmatrix_col,
+        )
         self.Nrmatrix.setRow(self.species)
         self.Nrmatrix.setCol(self.reactions)
 
-        self.Kmatrix = PyscesStoich.StructMatrix(self.__structural__.kmatrix, self.__structural__.kmatrix_row, self.__structural__.kmatrix_col)
+        self.Kmatrix = PyscesStoich.StructMatrix(
+            self.__structural__.kmatrix,
+            self.__structural__.kmatrix_row,
+            self.__structural__.kmatrix_col,
+        )
         self.Kmatrix.setRow(self.reactions)
         self.Kmatrix.setCol(self.reactions)
 
-        self.K0matrix = PyscesStoich.StructMatrix(self.__structural__.kzeromatrix, self.__structural__.kzeromatrix_row, self.__structural__.kzeromatrix_col)
+        self.K0matrix = PyscesStoich.StructMatrix(
+            self.__structural__.kzeromatrix,
+            self.__structural__.kzeromatrix_row,
+            self.__structural__.kzeromatrix_col,
+        )
         self.K0matrix.setRow(self.reactions)
         self.K0matrix.setCol(self.reactions)
 
-        self.Lmatrix = PyscesStoich.StructMatrix(self.__structural__.lmatrix, self.__structural__.lmatrix_row, self.__structural__.lmatrix_col)
+        self.Lmatrix = PyscesStoich.StructMatrix(
+            self.__structural__.lmatrix,
+            self.__structural__.lmatrix_row,
+            self.__structural__.lmatrix_col,
+        )
         self.Lmatrix.setRow(self.species)
         self.Lmatrix.setCol(self.species)
 
-        self.L0matrix = PyscesStoich.StructMatrix(self.__structural__.lzeromatrix, self.__structural__.lzeromatrix_row, self.__structural__.lzeromatrix_col)
+        self.L0matrix = PyscesStoich.StructMatrix(
+            self.__structural__.lzeromatrix,
+            self.__structural__.lzeromatrix_row,
+            self.__structural__.lzeromatrix_col,
+        )
         self.L0matrix.setRow(self.species)
         self.L0matrix.setCol(self.species)
 
         if self.__structural__.info_moiety_conserve:
-            self.Consmatrix = PyscesStoich.StructMatrix(self.__structural__.conservation_matrix, self.__structural__.conservation_matrix_row, self.__structural__.conservation_matrix_col)
+            self.Consmatrix = PyscesStoich.StructMatrix(
+                self.__structural__.conservation_matrix,
+                self.__structural__.conservation_matrix_row,
+                self.__structural__.conservation_matrix_col,
+            )
             self.Consmatrix.setRow(self.species)
             self.Consmatrix.setCol(self.species)
         else:
@@ -376,8 +482,9 @@ class PyscesInputFileParser(object):
         self.__StoichOK = 1
         print(' ')
 
+
 if __name__ == '__main__':
     ModelFile = 'pysces_test_linear1.psc'
-    ModelDir =  '/home/bgoli/Pysces/psc'
+    ModelDir = '/home/bgoli/Pysces/psc'
     mod = PyscesInputFileParser(ModelFile, ModelDir)
-    #~ mod.Stoichiometry_Analyse()
+    # ~ mod.Stoichiometry_Analyse()
